@@ -13,12 +13,13 @@ from jaaql.mvc.model import JAAQLModel
 from jaaql.mvc.response import JAAQLResponse
 from typing import Union
 
-from jaaql.openapi.swagger_documentation import SwaggerDocumentation, SwaggerMethod, TYPE__response,\
-    SwaggerFlatResponse, REST__DELETE, REST__GET, REST__OPTIONS, REST__POST, REST__PUT, SwaggerList, SwaggerResponse,\
+from jaaql.openapi.swagger_documentation import SwaggerDocumentation, SwaggerMethod, TYPE__response, \
+    SwaggerFlatResponse, REST__DELETE, REST__GET, REST__OPTIONS, REST__POST, REST__PUT, SwaggerList, SwaggerResponse, \
     MOCK__description, ARG_RESP__allow_all, RES__allow_all, SwaggerArgumentResponse, SwaggerSimpleList
 from jaaql.exceptions.http_status_exception import *
 
 ARG__http_inputs = "http_inputs"
+ARG__is_public = "is_public"
 ARG__sql_inputs = "sql_inputs"
 ARG__totp_iv = "totp_iv"
 ARG__user_id = "user_id"
@@ -54,6 +55,7 @@ ERR__unexpected_response_field = "Unexpected response field '%s'"
 ERR__method_required_security = "Method requires connection input yet marked as not secure in documentation"
 ERR__method_required_token = "Method requires oauth token input yet marked as not secure in documentation"
 ERR__method_required_totp = "Method requires totp iv input yet marked as not secure in documentation"
+ERR__method_required_is_public = "Method requires is_public input yet marked as not secure in documentation"
 ERR__method_required_user_id = "Method requires user id input yet marked as not secure in documentation"
 ERR__method_required_password_hash = "Method requires password hash input yet marked as not secure in documentation"
 ERR__missing_user_id = "Expected user id in response from method as method is without security"
@@ -108,9 +110,13 @@ class BaseJAAQLController:
             if isinstance(arg.arg_type, SwaggerList):
                 if not isinstance(data[arg.name], list):
                     raise HttpStatusException(ERR__argument_wrong_type % (arg.name, str(type(data[arg.name])),
-                                                                          str(type(list)), HTTPStatus.BAD_REQUEST))
+                                                                          str(type(list))), HTTPStatus.BAD_REQUEST)
                 for itm in data[arg.name]:
                     BaseJAAQLController.validate_data_rec(arg.arg_type.responses, itm, fill_missing)
+            elif arg.name in data and arg.arg_type == ARG_RESP__allow_all:
+                if not isinstance(data[arg.name], dict):
+                    raise HttpStatusException(ERR__argument_wrong_type % (arg.name, str(type(data[arg.name])),
+                                                                          str(type(dict))), HTTPStatus.BAD_REQUEST)
             elif arg.name in data and not isinstance(data[arg.name], arg.arg_type):
                 was_err = True
                 if arg.arg_type == int:
@@ -232,14 +238,15 @@ class BaseJAAQLController:
                         continue
                 if isinstance(real_resp[swag_resp.name], datetime):
                     real_resp[swag_resp.name] = str(real_resp[swag_resp.name])
-                if not is_complex and not isinstance(real_resp[swag_resp.name], swag_resp.arg_type):
-                    if real_resp[swag_resp.name] is not None or swag_resp.required:
-                        BaseJAAQLController.bi_cast(real_resp, swag_resp.name, swag_resp.arg_type)
+                if swag_resp.arg_type != ARG_RESP__allow_all:
+                    if not is_complex and not isinstance(real_resp[swag_resp.name], swag_resp.arg_type):
+                        if real_resp[swag_resp.name] is not None or swag_resp.required:
+                            BaseJAAQLController.bi_cast(real_resp, swag_resp.name, swag_resp.arg_type)
 
-                if is_complex:
-                    mock_resp = SwaggerResponse(MOCK__description, response=swag_resp.arg_type)
-                    real_resp[swag_resp.name] = BaseJAAQLController.validate_output(mock_resp,
-                                                                                    real_resp[swag_resp.name])
+                    if is_complex:
+                        mock_resp = SwaggerResponse(MOCK__description, response=swag_resp.arg_type)
+                        real_resp[swag_resp.name] = BaseJAAQLController.validate_output(mock_resp,
+                                                                                        real_resp[swag_resp.name])
 
                 match_alpha_resp[swag_resp.name] = real_resp[swag_resp.name]
             real_resp = match_alpha_resp
@@ -337,13 +344,15 @@ class BaseJAAQLController:
                     username = None
                     password_hash = None
                     l_totp = None
+                    is_public = None
 
                     ip_addr = request.headers.get(HEADER__real_ip, request.remote_addr).split(",")[0]
 
                     if swagger_documentation.security:
-                        jaaql_connection, user_id, ip_id, ua_id, totp_iv, password_hash, l_totp, username =\
+                        jaaql_connection, user_id, ip_id, ua_id, totp_iv, password_hash, l_totp, username, is_public = \
                             self.model.verify_jwt(request.headers.get(HEADER__security), ip_addr, user_agent,
-                            route == ENDPOINT__refresh)
+                                                  route == ENDPOINT__refresh,
+                                                  request.headers.get(HEADER__security_bypass))
 
                     supply_dict = {}
 
@@ -356,6 +365,11 @@ class BaseJAAQLController:
                                                                                                         self.is_prod)
                             method_input = self.log_safe_dump(supply_dict[ARG__http_inputs])
 
+                        if ARG__is_public in inspect.getfullargspec(view_func_local).args:
+                            if not swagger_documentation.security:
+                                raise Exception(ERR__method_required_is_public)
+                            supply_dict[ARG__is_public] = is_public
+
                         if ARG__sql_inputs in inspect.getfullargspec(view_func_local).args:
                             supply_dict[ARG__sql_inputs] = BaseJAAQLController.get_input_as_dictionary(
                                 method, self.is_prod, fill_missing=False)
@@ -437,7 +451,7 @@ class BaseJAAQLController:
                             if method.responses[0] == RES__allow_all:
                                 do_allow_all = True
 
-                        if not do_allow_all and ret_status != HTTPStatus.UNAUTHORIZED and ret_status !=\
+                        if not do_allow_all and ret_status != HTTPStatus.UNAUTHORIZED and ret_status != \
                                 HTTPStatus.NOT_IMPLEMENTED and ret_status != HTTPStatus.BAD_REQUEST and \
                                 ret_status != HTTPStatus.UNPROCESSABLE_ENTITY and \
                                 ret_status != CustomHTTPStatus.DATABASE_NO_EXIST and \
